@@ -33,6 +33,10 @@ class Snakefile:
         self._buffered_tokens.append(token)
 
 
+def comment_start(string: str) -> bool:
+    return string.lstrip().startswith("#")
+
+
 class Parser(ABC):
     def __init__(self, snakefile: TokenIterator):
         self.grammar = Grammar(
@@ -41,6 +45,7 @@ class Parser(ABC):
         self.context_stack = [self.grammar]
         self.snakefile: TokenIterator = snakefile
         self.from_python: bool = False
+        self.last_recognised_keyword: str = ""
 
         status = self.context.get_next_queriable(self.snakefile)
         self.buffer = status.buffer
@@ -52,7 +57,6 @@ class Parser(ABC):
             if status.eof:
                 break
 
-            self.from_python = False
             keyword = status.token.string
             if self.vocab.recognises(keyword):
                 if status.indent > self.target_indent:
@@ -60,19 +64,26 @@ class Parser(ABC):
                         self.from_python = True
                     else:  # Over-indented context gets reset
                         self.context.cur_indent = max(self.target_indent - 1, 0)
+                elif self.from_python:
+                    self.from_python = False
+                    # We are exiting python context, so force spacing out keywords
+                    self.last_recognised_keyword = ""
                 self.flush_buffer(
                     from_python=self.from_python,
                     in_global_context=self.in_global_context,
                 )
+                self.context.code_indent = None
                 status = self.process_keyword(status, self.from_python)
             else:
-                if not self.context.accepts_python_code and not keyword[0] == "#":
+                if not self.context.accepts_python_code and not comment_start(keyword):
                     raise SyntaxError(
                         f"L{status.token.start[0]}: Unrecognised keyword '{keyword}' "
                         f"in {self.context.keyword_name} definition"
                     )
                 else:
                     self.buffer += keyword
+                    if self.context.code_indent is None and not comment_start(keyword):
+                        self.context.code_indent = status.indent
                     status = self.context.get_next_queriable(self.snakefile)
                     self.buffer += status.buffer
             self.context.cur_indent = status.indent
